----------------------------------------------------------------------------------
-- Company: 
-- Engineer: Liam Gow
-- 
-- Create Date:    10:32:53 11/13/2014 
-- Design Name: 
-- Module Name:    MazeArray - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
-- Connects all subcomponents together
-- Holds and muxes all possible maze configurations
-- Takes in direction inputs, computes and updates resultant player LED positions
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity MazeArray is
    Port ( Direction : in  STD_LOGIC_VECTOR (3 downto 0);
			  Board : in STD_LOGIC_VECTOR (3 downto 0);
			  Reset : in STD_LOGIC;
			  CLK : in STD_LOGIC;
           Win : out  STD_LOGIC;
			  Power : out STD_LOGIC_VECTOR (15 downto 0);
			  Ground : out STD_LOGIC_VECTOR (15 downto 0);
			  Anodes : out STD_LOGIC_VECTOR (0 to 3);
			  Segments : out STD_LOGIC_VECTOR (0 to 7);
			  BuzzPin : out STD_LOGIC);
end MazeArray;

architecture Behavioral of MazeArray is

	-- component declarations

	component sseg_dec
    Port (      ALU_VAL : in std_logic_vector(7 downto 0); 
					    SIGN : in std_logic;
						VALID : in std_logic;
                    CLK : in std_logic;
                DISP_EN : out std_logic_vector(3 downto 0);
               SEGMENTS : out std_logic_vector(7 downto 0));
	end component;
	
	component Timing
		Port (	CLK : in  STD_LOGIC;
           TenthCLK : out  STD_LOGIC;
           SecondCLK : out  STD_LOGIC;
			  Iterate : out STD_LOGIC);
	end component;
	
	component Countdown
		Port ( secondCLK : in  STD_LOGIC;
					reset : in STD_LOGIC;
					win : in STD_LOGIC;
					limit : out STD_LOGIC_VECTOR (7 downto 0);
           timer : out  STD_LOGIC_VECTOR (7 downto 0));
	end component;
	
	component LEDDriver
		Port ( PlayerX : in  STD_LOGIC_VECTOR (7 downto 0);
             PlayerY : in  STD_LOGIC_VECTOR (7 downto 0);
					Power : out  STD_LOGIC_VECTOR (15 downto 0);
				  Ground : out  STD_LOGIC_VECTOR (15 downto 0));
	end component;
	
	component Buzzer
		Port ( Timer : in STD_LOGIC_VECTOR (7 downto 0);
			TenthCLK : in STD_LOGIC;
				Win : in STD_LOGIC;
				limit : in STD_LOGIC_VECTOR (7 downto 0);
           PinOut : out  STD_LOGIC);
	end component;
	
	-- Maze Array Declaration

	type maze_type is array (0 to 32, 0 to 32) of STD_LOGIC_VECTOR (2 downto 0);
	type coords is array (0 to 1) of std_logic_vector (7 downto 0);

	-- Signal Declaration

	signal maze : maze_type;
	shared variable player : coords;
	shared variable n : integer;

	signal player_sig : coords;
	signal TenthCLK : STD_LOGIC := '0';
	signal secondCLK : STD_LOGIC := '0';
	signal Iterate : STD_LOGIC := '0';
	signal sseg_data : std_logic_vector (7 downto 0);
	
	signal playerX : std_logic_vector (7 downto 0);
	signal playerY : std_logic_vector (7 downto 0);
	
	signal win_cond : std_logic := '0';
	
	signal limit : std_logic_vector (7 downto 0);
	
	

begin
	
	-- Connection declarations
	
	SSEG : sseg_dec
	port map ( 	ALU_VAL => sseg_data,
					SIGN => '0',
					VALID => '1',
					CLK => CLK,
					DISP_EN => Anodes,
					SEGMENTS => Segments);
					
	Clocks : Timing
	port map ( CLK => CLK,
          TenthCLK => TenthCLK,
        SecondCLK => secondCLK,
		  Iterate => Iterate);
		  
	Counter : Countdown
	port map (  reset => reset,
					secondCLK => SecondCLK,
					win => win_cond,
					limit => limit,
						timer => sseg_data);
						
	LEDs : LEDDriver
	port map (  PlayerX => playerX,
					PlayerY => playerY,
					  Power => Power,
					 Ground => Ground);
					 
	Beep : Buzzer
	port map ( 	Timer => sseg_data,
				TenthCLK => TenthCLK,
				Win => win_cond,
				limit => limit,
					PinOut => BuzzPin);
					 
	playerX <= player_sig(1);
	playerY <= player_sig(0);

	win <= win_cond;
	
	
	-- LED position iterator
	
update_board : process (Iterate, reset, board, win_cond)
	variable new_player : coords;
	variable potential_wall : coords;
begin
	if rising_edge(iterate) then
	
	--	Finds potential player positions and potential obstructions based on input direction
		if reset = '0' and win_cond = '0' and (not (sseg_data = "00000000")) then -- If you want to be able to move after the timer runs out, remove 'and not (sseg_data = "00000000")'
			case direction is
				when "0001" => -- Right
					new_player := (player(0), player(1)+X"2");
					potential_wall := (player(0), player(1)+'1');
				when "0010" => -- Down
					new_player := (player(0)+X"2", player(1));
					potential_wall := (player(0)+'1', player(1));
				when "0100" => -- Left
					new_player := (player(0), player(1)-X"2");
					potential_wall := (player(0), player(1)-'1');
				when "1000" => -- Up
					new_player := (player(0)-X"2", player(1));
					potential_wall := (player(0)-'1', player(1));
				when others => -- Null
					new_player := player;
					potential_wall := player;
			end case;
		
		-- Overwrites player position with new position if no obstruction is present
			if maze(to_integer(unsigned(potential_wall(0))), (to_integer(unsigned(potential_wall(1))))) = "000" then
				player := new_player;
			elsif maze(to_integer(unsigned(potential_wall(0))), (to_integer(unsigned(potential_wall(1))))) = "101" then
				win_cond <= '1';
			end if;
		
		-- Initializes player initial states based on board input
		elsif (to_integer(unsigned(sseg_data))) = to_integer(unsigned(limit)) - 1 then -- Countdown Timer-1
			case board is
				when "0000" =>
					player := (std_logic_vector(to_unsigned((1), 8)), std_logic_vector(to_unsigned((15), 8)));
				when others =>
					player := (std_logic_vector(to_unsigned((1), 8)), std_logic_vector(to_unsigned((19), 8)));
			end case;
		-- Resets win signal on reset
		elsif reset = '1' then
			win_cond <= '0';
		end if;
	end if;
	
	-- assigns temporary player signal to output
	player_sig <= player;
end process;


-- selects board configuration from board input
with board select maze <=

	-- Symbols are suggested for use in the maze array initialization, to be Replaced (Ctrl+H) with their binary values

	-- 000 : ! : 0 is a open wall space
	-- 001 : @ : 1 is a wall
	-- 010 : # : 2 is a open player space
	-- 011 : $ : 3 is a player
	-- 100 : % : 4 is the starting point
	-- 101 : ^ : 5 is the ending point

	-- Maze Array Assignments
	  (("001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","100","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","010","001","010","000","010","000","010","000","010","001","010","000","010","001","010","000","010","000","010","000","010","000","010","000","010","001","010","000","010","001","001","001"),
		("001","000","001","000","001","001","001","001","001","000","001","000","001","001","001","000","001","001","001","001","001","001","001","001","001","000","001","000","001","000","001","001","001"),
		("001","010","000","010","001","010","000","010","001","010","000","010","000","010","000","010","001","010","000","010","000","010","001","010","001","010","001","010","001","010","001","001","001"),
		("001","001","001","000","001","000","001","000","001","001","001","001","001","000","001","001","001","000","001","001","001","000","001","000","001","000","001","000","001","001","001","001","001"),
		("001","010","000","010","000","010","001","010","000","010","001","010","000","010","001","010","000","010","001","010","000","010","000","010","000","010","001","010","000","010","001","001","001"),
		("001","000","001","001","001","001","001","001","001","000","001","000","001","001","001","000","001","001","001","001","001","001","001","001","001","000","001","001","001","000","001","001","001"),
		("001","010","000","010","001","010","000","010","000","010","001","010","000","010","001","010","000","010","001","010","000","010","000","010","001","010","000","010","001","010","001","001","001"),
		("001","001","001","000","001","001","001","001","001","001","001","001","001","001","001","000","001","000","001","000","001","001","001","000","001","001","001","001","001","000","001","001","001"),
		("001","010","001","010","000","010","000","010","001","010","000","010","000","010","000","010","001","010","000","010","001","010","000","010","001","010","000","010","000","010","001","001","001"),
		("001","000","001","001","001","001","001","000","001","000","001","001","001","001","001","000","001","001","001","001","001","001","001","001","001","000","001","000","001","001","001","001","001"),
		("001","010","000","010","001","010","000","010","001","010","000","010","000","010","001","010","001","010","000","010","000","010","000","010","000","010","001","010","000","010","001","001","001"),
		("001","000","001","000","001","000","001","001","001","001","001","001","001","001","001","000","001","000","001","001","001","001","001","000","001","001","001","000","001","001","001","001","001"),
		("001","010","001","010","000","010","001","010","000","010","001","010","000","010","000","010","001","010","001","010","000","010","001","010","000","010","001","010","000","010","001","001","001"),
		("001","001","001","001","001","001","001","000","001","000","001","000","001","001","001","000","001","000","001","000","001","000","001","001","001","000","001","001","001","000","001","001","001"),
		("001","010","000","010","000","010","000","010","001","010","000","010","000","010","001","010","001","010","001","010","001","010","000","010","001","010","000","010","001","010","001","001","001"),
		("001","000","001","001","001","001","001","001","001","001","001","001","001","000","001","000","001","000","001","000","001","001","001","000","001","000","001","001","001","000","001","001","001"),
		("001","010","001","010","000","010","000","010","000","010","000","010","000","010","001","010","001","010","001","010","000","010","001","010","000","010","001","010","000","010","001","001","001"),
		("001","000","001","000","001","001","001","000","001","001","001","001","001","001","001","001","001","000","001","001","001","001","001","001","001","001","001","000","001","001","001","001","001"),
		("001","010","001","010","000","010","001","010","000","010","000","010","001","010","000","010","001","010","000","010","001","010","000","010","000","010","000","010","000","010","001","001","001"),
		("001","001","001","001","001","000","001","000","001","001","001","001","001","000","001","000","001","000","001","000","001","000","001","001","001","001","001","001","001","000","001","001","001"),
		("001","010","000","010","000","010","001","010","001","010","000","010","000","010","001","010","000","010","001","010","001","010","000","010","000","010","001","010","000","010","001","001","001"),
		("001","000","001","001","001","001","001","001","001","000","001","001","001","001","001","000","001","001","001","001","001","001","001","001","001","000","001","000","001","001","001","001","001"),
		("001","010","000","010","001","010","000","010","000","010","000","010","000","010","001","010","001","010","001","010","000","010","000","010","000","010","001","010","000","010","001","001","001"),
		("001","001","001","000","001","000","001","001","001","001","001","001","001","001","001","000","001","000","001","000","001","000","001","001","001","001","001","001","001","000","001","001","001"),
		("001","010","000","010","001","010","000","010","001","010","000","010","000","010","000","010","000","010","001","010","001","010","001","010","000","010","000","010","001","010","001","001","001"),
		("001","000","001","001","001","000","001","000","001","000","001","001","001","000","001","001","001","001","001","000","001","000","001","001","001","000","001","001","001","000","001","001","001"),
		("001","010","001","010","000","010","001","010","001","010","001","010","001","010","000","010","000","010","001","010","001","010","000","010","001","010","000","010","001","010","001","001","001"),
		("001","000","001","000","001","001","001","000","001","000","001","000","001","001","001","001","001","001","001","000","001","001","001","000","001","000","001","000","001","000","001","001","001"),
		("001","010","000","010","001","010","000","010","001","010","000","010","000","010","001","010","000","010","000","010","001","010","000","010","001","010","001","010","000","010","001","001","001"),
		("001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","101","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001"))
		
		when "0000",
		
	  (("001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","010","000","010","000","010","000","010","000","010","000","010","000","010","000","010","000","010","000","010","100","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","000","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","000","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","010","001","010","000","010","000","010","000","010","000","010","001","010","000","010","000","010","000","010","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","000","001","001","001","001","001","000","001","001","001","000","001","000","001","001","001","001","001","000","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","010","001","010","000","010","001","010","000","010","001","010","001","010","000","010","001","010","000","010","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","000","001","000","001","000","001","001","001","000","001","000","001","001","001","000","001","001","001","000","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","010","001","010","001","010","001","010","000","010","001","010","000","010","000","010","001","010","000","010","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","000","001","000","001","000","001","000","001","001","001","001","001","001","001","001","001","000","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","010","001","010","001","010","001","010","001","010","000","010","000","010","001","010","000","010","000","010","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","000","001","000","001","000","001","000","001","000","001","001","001","000","001","000","001","001","001","000","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","010","001","010","001","010","001","010","000","010","001","010","000","010","001","010","000","010","001","010","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","000","001","001","001","000","001","001","001","000","001","001","001","001","001","001","001","001","001","000","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","010","000","010","000","010","000","010","001","010","000","010","000","010","001","010","000","010","001","010","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","001","001","001","001","001","001","001","001","001","001","001","001","000","001","001","001","000","001","000","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("101","010","000","010","000","010","000","010","000","010","000","010","000","010","001","010","000","010","000","010","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001"),
		("001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001","001"))

		when others;

end Behavioral;



